(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: upph_id_card.ml,v 1.12 2014/02/09 17:27:28 fclement Exp $ *)

(* Handling the user part of the id card of a Pph project. *)

let pic_qual = Upervasives.qualify "Upph_id_card";;

let pic_failwith = Upervasives.failwith_qual pic_qual;;

open Ucommand_option;;

type pph_id_card = {
  project_name : string;
  project_banner_title : string;
  project_outline : string;
  project_release_date : string;
  project_copyright_handlers : string;
  project_authors : string;
  project_http_url : string;
  project_ftp_url : string;
  project_release_ident : string;
  project_distribution_ident : string;
  project_version : string;
  project_version_full_info : string;
  project_release_name : string;
  project_distribution_name : string;
}
;;

type t = pph_id_card;;

(* Print version information. *)

let make_print_version name version pic =
  let line = Printf.sprintf "%s, version %s" (name pic) (version pic) in
  (fun () ->
    prerr_endline line;
    exit 0)
;;

let short_name pic = pic.project_name;;
let long_name pic =
  Printf.sprintf "%s, %s" pic.project_name pic.project_banner_title
;;

let short_version pic = pic.project_release_ident
and long_version pic = pic.project_version_full_info
;;

let make_print_short_version = make_print_version short_name short_version
and make_print_long_version = make_print_version long_name long_version
;;

(* Adding command line options. *)

let short_option_name pic =
  let sn = pic.project_name in
  if String.length sn = 0
  then pic_failwith "short_option_name" "empty project name"
  else sn.[0]
;;

let add_user_version_options pic =
  Ucommand_option.user_add_all
    [
     Ucommand_option.of_regular
       {
        key = "--version";
        spec = Arg.Unit (make_print_long_version pic);
        doc =
          Printf.sprintf
            " %s detailed version information."
            pic.project_name;
       };
     
     Ucommand_option.of_regular
       {
        key = "-v";
        spec = Arg.Unit (make_print_short_version pic);
        doc =
          Printf.sprintf
            " %s short version information."
            pic.project_name;
       };
   ]
;;
    
let add_library_version_options pic =
  Ucommand_option.library_add_all
    [
     Ucommand_option.of_regular
       {
        key = Printf.sprintf "--%s-version" pic.project_name;
        spec = Arg.Unit (make_print_long_version pic);
        doc =
          Printf.sprintf
            " %s detailed version information."
            pic.project_name;
       };
     
     Ucommand_option.of_regular
       {
        key = Printf.sprintf "-%cv" (short_option_name pic);
        spec = Arg.Unit (make_print_short_version pic);
        doc = 
          Printf.sprintf
            " %s short version information."
            pic.project_name;
       };
   ]
;;
